@@ -9,7 +9,7 @@ use Carp;
 use Config;
 use Math::DCT 'dct2d';
 
-our $VERSION = '0.2_1';
+our $VERSION = '0.2_2';
 
 =head1 NAME
 
@@ -68,7 +68,8 @@ Depending on your setup, it should be 15x or so faster than pHash.org
   my $iph = Image::PHash->new($image_file , $library?, \%settings?);
   
 The first argument is an image filename when the Imlib2 library is used, but it
-can also by a variable with the image data for the other libraries.
+can also by a variable with the image data for the other libraries, or even an
+image object for the supported libraries.
 
 The second (optional) argument is the image library. Valid options are C<Imlib2>,
 C<GD>, C<Imager>, C<ImageMagick> and if not specified the module will try to load
@@ -408,9 +409,20 @@ sub new {
         Imager      => ['Imager', \&_reduce_Imager, \&_pixels_Imager, 1],
     );
 
-    my $file = -f $self->{image};
+    my $ref  = ref($self->{image});
+    my $file = $ref ? undef : -f $self->{image};
 
-    if ($self->{module}) {    # User specified image module
+    if ($ref) {    # Passed image object
+        $self->{module} = undef;
+        foreach (_lib_order()) {
+            my $type = $_ eq 'GD' ? 'GD::Image' : $libs{$_}->[0];
+            if ($type eq $ref) {
+                $self->{module} = $_;
+                last;
+            }
+        }
+        croak("Object of unknown type $ref.") unless $self->{module};
+    } elsif ($self->{module}) {    # User specified image module
         $libs{$libs{$_}->[0]} = $libs{$_} for keys %libs;    # Allow synonyms
         my $lib = $libs{$self->{module}}->[0];
 
@@ -423,8 +435,8 @@ sub new {
 
         $self->{module} = $lib; # Normalize
     } else {
-        # Try to load Imlib2, GD, Imager, ImageMagic in that order
-        foreach my $lib (qw/Imlib2 GD Imager Magick/) {
+        # Try to load Imlib2, GD, Imager, ImageMagick in that order
+        foreach my $lib (_lib_order()) {
             if (eval "require $libs{$lib}->[0]" && ($file || $libs{$lib}->[3])) {
                 $self->{module} = $lib;
                 last;
@@ -435,24 +447,28 @@ sub new {
         ) unless $self->{module};
     }
     croak("No file at $self->{image}")
-        unless $file || (length($self->{image}) > 255 && $libs{$self->{module}}->[3]);
+        unless $ref || $file || (length($self->{image}) > 255 && $libs{$self->{module}}->[3]);
 
     my $error = '';
     if ($libs{$self->{module}}->[0] eq 'Image::Imlib2') {
-        $self->{im} = Image::Imlib2->load($self->{image});
+        $self->{im} = $ref ? $self->{image} : Image::Imlib2->load($self->{image});
     } elsif ($libs{$self->{module}}->[0] eq 'GD') {
         GD::Image->trueColor(1);
-        $self->{im} = GD::Image->new($self->{image});
+        $self->{im} = $ref ? $self->{image} : GD::Image->new($self->{image});
     } elsif ($libs{$self->{module}}->[0] eq 'Image::Magick') {
-        $self->{im} = Image::Magick->new();
-        $error =
-            ($file)
-            ? $self->{im}->Read($self->{image})
-            : $self->{im}->BlobToImage($self->{image});
-        $self->{im} = undef if $error;
+        if ($ref) {
+            $self->{im} = $self->{image};
+        } else {
+            $self->{im} = Image::Magick->new();
+            $error =
+                ($file)
+                ? $self->{im}->Read($self->{image})
+                : $self->{im}->BlobToImage($self->{image});
+            $self->{im} = undef if $error;
+        }
     } else {
         my $type = $file ? 'file' : 'data';
-        $self->{im} = Imager->new($type => $self->{image});
+        $self->{im} = $ref ? $self->{image} : Imager->new($type => $self->{image});
         $error = Imager->errstr() || '';
     }
 
@@ -497,6 +513,8 @@ sub b2h {
     return $hex;
 }
 
+sub _lib_order {qw/Imlib2 GD Imager ImageMagick/}
+
 sub _is_64bit {
     $_64bit //= $Config{ivsize} >= 8;
     return $_64bit;
@@ -869,8 +887,8 @@ sub _apply_median {
 
 =head1 ACKNOWLEDGEMENTS
 
-Initially based on L<Image::Hash>, so code to do with loading images, pixels etc
-has been kept.
+Initially based on L<Image::Hash>, so some code to do with loading images, pixels
+etc has been kept/adapted.
 
 =head1 AUTHOR
 
